import hopsworks
import json
import hsfs.constructor.query
import logging

logger = logging.getLogger('logger')


class FeatureStoreService:
    def __init__(self, config):
        self.conf = config
        with open(self.conf, "r") as jsonfile:
            data = json.load(jsonfile)

        self.hopsworks_host = data['hopsworks_host']
        self.hopsworks_port = data['hopsworks_port']
        self.hopsworks_project = data['hopsworks_project']
        self.hopsworks_api_key_value = data['hopsworks_api_key_value']

    def get_feature_store_connection(self):
        try:
            logger.info('Trying to establish a connection with the hopsworks project')
            connection = hopsworks.login(host=self.hopsworks_host,
                                         port=self.hopsworks_port,
                                         project=self.hopsworks_project,
                                         api_key_value=self.hopsworks_api_key_value)
            fs = connection.get_feature_store()
            logger.info('Connection successfully established with the hopsworks project')
            return fs

        except Exception as e:
            logger.error('Connection failed to establish. ' + e)

        finally:
            pass

    def close_feature_store_connection(self, fs):
        try:
            hopsworks.Connection.close(fs)
            logger.info('Connection closed with the hopsworks project')
        except Exception as e:
            logger.error(e)

        finally:
            pass

    def get_group(self, group_name: str, version: int):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to retrieve a feature group')
            fg = fs.get_feature_group(group_name, version)
            logger.info('Feature group has been successfully retrieved')
            return fg

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def build_group(self, group_name: str, version: int, description: str, primary_key: [str], partition_key: [str],
                    features: [hsfs.feature.Feature], df):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to build a feature group')
            fg = fs.get_or_create_feature_group(
                name=group_name,
                version=version,
                description=description,
                primary_key=primary_key,
                partition_key=partition_key
            )
            fg.insert(df)
            logger.info('Feature group has been successfully built')

        except Exception as e:
            logger.error(e)
            pass

        finally:
            pass

    def drop_group(self, group_name: str, version: int):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to delete a feature group')
            fg = fs.get_feature_group(group_name, version)
            fg.delete()
            logger.info('Feature group has been successfully deleted')

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def get_view(self, view_name: str, version: int):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to retrieve a feature view')
            fv = fs.get_feature_view(view_name, version)
            logger.info('Feature view has been successfully retrieved')
            return fv

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def build_view(self, view_name: str, view_version: int, description: str, trans_functions: [],
                   query: hsfs.constructor.query.Query):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to build a feature view')
            logger.info('Retrieving the respective feature group')

            fs.create_feature_view(
                name=view_name,
                version=view_version,
                description=description,
                ## TODO: insert the correct labels below
                labels=["box_type"],
                query=query
            )
            logger.info('Feature view has been successfully created')

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def drop_view(self, view_name: str, version: int):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to delete a feature view')
            fv = fs.get_feature_view(view_name, version)
            fv.delete()
            logger.info('Feature view has been successfully deleted')

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def do_drop_group(self, group_name: str, version: int):
        try:
            fs = self.get_feature_store_connection()
            logger.info('Trying to hard delete a feature group')
            logger.info('Feature group has been successfully hard deleted')

        except Exception as e:
            logger.error(e)

        finally:
            pass



import hsfs.constructor.query
import logging
import pandas as pd
from hsfs.feature import Feature
import json
import sys
import os

parent_dir = os.path.dirname(os.getcwd())
sys.path.insert(0,parent_dir)

from src.service import FeatureStoreService as fss
from src.utils import multi_melt as melt
logger = logging.getLogger('logger')


class ComplexFeatureService:
    def __init__(self, config):
        self.conf = config
        with open(self.conf, "r") as jsonfile:
            data = json.load(jsonfile)

    def build_view_team(self):
        try:
            logger.info('Getting feature groups details')
            features_game_base = ["uuid", "League", "Date", "Team1", "Team2", "Season", "G_Type", "T1_loc", "T2_loc",
                                  "T1_sea_g_played", "T1_sea_g_played_h", "T1_sea_g_played_a", "T2_sea_g_played",
                                  "T2_sea_g_played_h", "T2_sea_g_played_a", "T1_B2B", "T2_B2B"]
            features_box_score = ["T1_pts", "T2_pts", "T1_Result", "T2_Result", "Box_Type", "MP", "T1_FGM", "T1_FGA",
                                  "T1_FG_Pct", "T1_2PM", "T1_2PA", "T1_2P_Pct", "T1_3PM", "T1_3PA", "T1_3P_Pct",
                                  "T1_FTM", "T1_FTA", "T1_FT_Pct", "T1_OREB", "T1_DREB", "T1_REB", "T1_AST", "T1_TOV",
                                  "T1_STL", "T1_BLK", "T1_PF", "T2_FGM", "T2_FGA", "T2_FG_Pct", "T2_2PM", "T2_2PA",
                                  "T2_2P_Pct", "T2_3PM", "T2_3PA", "T2_3P_Pct", "T2_FTM", "T2_FTA", "T2_FT_Pct",
                                  "T2_OREB", "T2_DREB", "T2_REB", "T2_AST", "T2_TOV", "T2_STL", "T2_BLK", "T2_PF",
                                  "PD_Box", "PT_Box"]
            features_adv_stats = ["T1_Poss_Cnt", "T1_Poss_Sec", "T1_Poss_Sdp", "Box_Type", "T1_FGA_Poss_Prop",
                                  "T1_2PA_Poss_Prop", "T1_3PA_Poss_Prop", "T1_FTA_Poss_Prop", "T1_FTA_FGA_Rto",
                                  "T1_EFG_Pct", "T1_TS_Pct", "T1_OREB_REB_Pct", "T1_DREB_REB_Pct", "T1_TOV_Poss_Pct",
                                  "T1_PF_Poss_Pct", "T2_Poss_Cnt", "T2_Poss_Sec", "T2_Poss_Sdp", "T2_FGA_Poss_Prop",
                                  "T2_2PA_Poss_Prop", "T2_3PA_Poss_Prop", "T2_FTA_Poss_Prop", "T2_FTA_FGA_Rto",
                                  "T2_EFG_Pct", "T2_TS_Pct", "T2_OREB_REB_Pct", "T2_DREB_REB_Pct", "T2_TOV_Poss_Pct",
                                  "T2_PF_Poss_Pct", "PD_Poss", "PT_Poss"]
            fss_obj = fss.FeatureStoreService(self.conf)
            fg_game_base = fss_obj.get_group(
                "gp_game_base",
                2
            )

            fg_box_score = fss_obj.get_group(
                "gp_box_score",
                1
            )

            fg_adv_stats = fss_obj.get_group(
                "gp_adv_stats",
                1
            )

            df = fg_box_score.select(features_box_score).filter(Feature('uuid') == 6568) \
                .join(fg_adv_stats.select(features_adv_stats).filter(Feature('uuid') == 6568), \
                      on=['uuid', 'box_type'], join_type='inner') \
                .join(fg_game_base.select(features_game_base).filter(Feature('uuid') == 6568), \
                      on=['uuid'], join_type='inner') \
                .read()

            df['t1_pd_box'] = df['t2_pts'] - df['t1_pts']
            df['t2_pd_box'] = df['t1_pts'] - df['t2_pts']

            df['t1_pd_poss'] = (df['t2_pts'] - df['t1_pts']) / (df[['t1_poss_cnt', 't2_poss_cnt']].mean(axis=1))
            df['t2_pd_poss'] = (df['t1_pts'] - df['t2_pts']) / (df[['t1_poss_cnt', 't2_poss_cnt']].mean(axis=1))

            id_vars = ['uuid', 'date', 'season', 'box_type', 'g_type', 'mp', 'pt_box', 'pt_poss']

            value_vars = [['Team1', 'Team2'], ['T1_Loc', 'T2_Loc'], ['T1_sea_g_played', 'T2_sea_g_played'],
                          ['T1_B2B', 'T2_B2B'], ['t1_pd_box', 't2_pd_box'], ['t1_pd_poss', 't2_pd_poss'],
                          ['T1_PTS', 'T2_PTS'], ['T1_FGM', 'T2_FGM'], ['T1_FGA', 'T2_FGA'],
                          ['T1_FG_Pct', 'T2_FG_Pct'], ['T1_2PM', 'T2_2PM'], ['T1_2PA', 'T2_2PA'],
                          ['T1_2P_Pct', 'T2_2P_Pct'],
                          ['T1_3PM', 'T2_3PM'], ['T1_3PA', 'T2_3PA'], ['T1_3P_Pct', 'T2_3P_Pct'], ['T1_FTM', 'T2_FTM'],
                          ['T1_FTA', 'T2_FTA'], ['T1_FT_Pct', 'T2_FT_Pct'], ['T1_OREB', 'T2_OREB'],
                          ['T1_DREB', 'T2_DREB'],
                          ['T1_REB', 'T2_REB'], ['T1_AST', 'T2_AST'], ['T1_TOV', 'T2_TOV'], ['T1_STL', 'T2_STL'],
                          ['T1_BLK', 'T2_BLK'],
                          ['T1_PF', 'T2_PF'], ['T2_PTS', 'T1_PTS'], ['T2_FGM', 'T1_FGM'], ['T2_FGA', 'T1_FGA'],
                          ['T2_FG_Pct', 'T1_FG_Pct'], ['T2_2PM', 'T1_2PM'], ['T2_2PA', 'T1_2PA'],
                          ['T2_2P_Pct', 'T1_2P_Pct'],
                          ['T2_3PM', 'T1_3PM'], ['T2_3PA', 'T1_3PA'], ['T2_3P_Pct', 'T1_3P_Pct'], ['T2_FTM', 'T1_FTM'],
                          ['T2_FTA', 'T1_FTA'], ['T2_FT_Pct', 'T1_FT_Pct'], ['T2_OREB', 'T1_OREB'],
                          ['T2_DREB', 'T1_DREB'],
                          ['T2_REB', 'T1_REB'], ['T2_AST', 'T1_AST'], ['T2_TOV', 'T1_TOV'], ['T2_STL', 'T1_STL'],
                          ['T2_BLK', 'T1_BLK'],
                          ['T2_PF', 'T1_PF'], ['T1_Poss_Cnt', 'T2_Poss_Cnt'], ['T1_Poss_Sec', 'T2_Poss_Sec'],
                          ['T1_Poss_Sdp', 'T2_Poss_Sdp'], ['T1_FGA_Poss_Prop', 'T2_FGA_Poss_Prop'],
                          ['T1_2PA_Poss_Prop', 'T2_2PA_Poss_Prop'], ['T1_3PA_Poss_Prop', 'T2_3PA_Poss_Prop'],
                          ['T1_FTA_Poss_Prop', 'T2_FTA_Poss_Prop'], ['T1_FTA_FGA_Rto', 'T2_FTA_FGA_Rto'],
                          ['T1_EFG_Pct', 'T2_EFG_Pct'],
                          ['T1_TS_Pct', 'T2_TS_Pct'], ['T1_OREB_REB_Pct', 'T2_OREB_REB_Pct'],
                          ['T1_DREB_REB_Pct', 'T2_DREB_REB_Pct'],
                          ['T1_TOV_Poss_Pct', 'T2_TOV_Poss_Pct'], ['T1_PF_Poss_Pct', 'T2_PF_Poss_Pct'],
                          ['T2_Poss_Cnt', 'T1_Poss_Cnt'],
                          ['T2_Poss_Sec', 'T1_Poss_Sec'], ['T2_Poss_Sdp', 'T1_Poss_Sdp'],
                          ['T2_FGA_Poss_Prop', 'T1_FGA_Poss_Prop'],
                          ['T2_2PA_Poss_Prop', 'T1_2PA_Poss_Prop'], ['T2_3PA_Poss_Prop', 'T1_3PA_Poss_Prop'],
                          ['T2_FTA_Poss_Prop', 'T1_FTA_Poss_Prop'], ['T2_FTA_FGA_Rto', 'T1_FTA_FGA_Rto'],
                          ['T2_EFG_Pct', 'T1_EFG_Pct'],
                          ['T2_TS_Pct', 'T1_TS_Pct'], ['T2_OREB_REB_Pct', 'T1_OREB_REB_Pct'],
                          ['T2_DREB_REB_Pct', 'T1_DREB_REB_Pct'],
                          ['T2_TOV_Poss_Pct', 'T1_TOV_Poss_Pct'], ['T2_PF_Poss_Pct', 'T1_PF_Poss_Pct']]

            value_name = ['Team', 'Team_Loc', 'Team_sea_g_played', 'Team_B2B', 'PD_Box', 'PD_Poss', 'Off_PTS',
                          'Off_FGM',
                          'Off_FGA', 'Off_FG_Pct', 'Off_2PM', 'Off_2PA', 'Off_2P_Pct', 'Off_3PM', 'Off_3PA',
                          'Off_3P_Pct',
                          'Off_FTM', 'Off_FTA', 'Off_FT_Pct', 'Off_OREB', 'Off_DREB', 'Off_REB', 'Off_AST', 'Off_TOV',
                          'Off_STL', 'Off_BLK', 'Off_PF', 'Def_PTS', 'Def_FGM',
                          'Def_FGA', 'Def_FG_Pct', 'Def_2PM', 'Def_2PA', 'Def_2P_Pct', 'Def_3PM', 'Def_3PA',
                          'Def_3P_Pct',
                          'Def_FTM', 'Def_FTA', 'Def_FT_Pct', 'Def_OREB', 'Def_DREB', 'Def_REB', 'Def_AST', 'Def_TOV',
                          'Def_STL', 'Def_BLK', 'Def_PF', 'Off_Poss_Cnt', 'Off_Poss_Sec', 'Off_Poss_Sdp',
                          'Off_FGA_Poss_Prop',
                          'Off_2PA_Poss_Prop', 'Off_3PA_Poss_Prop', 'Off_FTA_Poss_Prop', 'Off_FTA_FGA_Rto',
                          'Off_EFG_Pct',
                          'Off_TS_Pct', 'Off_OREB_REB_Pct', 'Off_DREB_REB_Pct', 'Off_TOV_Poss_Pct', 'Off_PF_Poss_Pct',
                          'Def_Poss_Cnt', 'Def_Poss_Sec', 'Def_Poss_Sdp', 'Def_FGA_Poss_Prop', 'Def_2PA_Poss_Prop',
                          'Def_3PA_Poss_Prop', 'Def_FTA_Poss_Prop', 'Def_FTA_FGA_Rto', 'Def_EFG_Pct', 'Def_TS_Pct',
                          'Def_OREB_REB_Pct', 'Def_DREB_REB_Pct', 'Def_TOV_Poss_Pct', 'Def_PF_Poss_Pct']

            test = [[x.lower() for x in sub_value_vars] for sub_value_vars in value_vars]

            df_melted = melt.call_multi_melt(df,
                                             [x.lower() for x in id_vars],
                                             [[x.lower() for x in sub_value_vars] for sub_value_vars in value_vars],
                                             [x.lower() for x in value_name]).drop(columns=['variable'])

            pd.set_option('display.max_columns', None)
            pd.set_option('display.width', 1000)

            final_df = df_melted[
                ['uuid', 'date', 'season', 'g_type', 'team', 'team_loc', 'team_sea_g_played', 'team_b2b', 'mp',
                 'box_type', 'pd_box', 'pt_box', 'off_pts', 'off_fgm', 'off_fga', 'off_fg_pct', 'off_2pm', 'off_2pa',
                 'off_2p_pct', 'off_3pm', 'off_3pa', 'off_3p_pct', 'off_ftm', 'off_fta', 'off_ft_pct', 'off_oreb',
                 'off_dreb', 'off_reb', 'off_ast', 'off_tov', 'off_stl', 'off_blk', 'off_pf', 'def_pts', 'def_fgm',
                 'def_fga', 'def_fg_pct', 'def_2pm', 'def_2pa', 'def_2p_pct', 'def_3pm', 'def_3pa', 'def_3p_pct',
                 'def_ftm', 'def_fta', 'def_ft_pct', 'def_oreb', 'def_dreb', 'def_reb', 'def_ast', 'def_tov',
                 'def_stl', 'def_blk', 'def_pf', 'pd_poss', 'pt_poss', 'off_poss_cnt', 'off_poss_sec', 'off_poss_sdp',
                 'off_fga_poss_prop', 'off_2pa_poss_prop', 'off_3pa_poss_prop', 'off_fta_poss_prop',
                 'off_fta_fga_rto', 'off_efg_pct', 'off_ts_pct', 'off_oreb_reb_pct', 'off_dreb_reb_pct',
                 'off_tov_poss_pct', 'off_pf_poss_pct', 'def_poss_cnt', 'def_poss_sec', 'def_poss_sdp',
                 'def_fga_poss_prop', 'def_2pa_poss_prop', 'def_3pa_poss_prop', 'def_fta_poss_prop',
                 'def_fta_fga_rto', 'def_efg_pct', 'def_ts_pct', 'def_oreb_reb_pct', 'def_dreb_reb_pct',
                 'def_tov_poss_pct', 'def_pf_poss_pct']]
            final_df.sort_values(by=['uuid', 'box_type'], inplace=True)

            filtered_df = final_df[['uuid', 'team', 'box_type', 'pd_box', 'pt_box', 'pd_poss', 'pt_poss']]
            print(filtered_df)

            fss_obj.build_group('temp_group',
                                1,
                                "temporary feature group",
                                ['uuid', 'date', 'season', 'box_type', 'team'],
                                [],
                                final_df.columns.tolist(),
                                final_df)
            fg_temp = fss_obj.get_group('temp_group', 1)
            query = fg_temp.select_all()
            fss_obj.drop_group('temp_group', 1)
            return query

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def build_view_t1vst2(self, view_name, view_version, description, view_json):
        try:
            logger.info('Getting feature groups details')
            query = hsfs.constructor.query.Query

            for groups in view_json:
                name = groups["name"]
                version = groups["version"]
                features = groups["features"]
                join_sequence = groups["join_sequence"]
                joining_columns = groups["joining_columns"]
                join_type = groups["join_type"]
                fss_obj = fss.FeatureStoreService(self.conf)
                fg_name = fss_obj.get_group(
                    name,
                    version
                )
                filters = ""
                if join_sequence == 1:
                    if groups["filters"] is not str.strip(""):
                        filters = ".filter(" + groups["filters"] + ")"
                    q = "fg_name.select([" + "'{}'".format("','".join(features)) + "])" + filters
                    query = eval(q)
                else:
                    if groups["filters"] is not str.strip(""):
                        filters = ".filter(" + groups["filters"] + ")"
                    q = "query.join(fg_name.select([" + "'{}'".format(
                        "','".join(features)) + "])" + filters + ",on=[" + "'{}'".format(
                        "','".join(joining_columns)) + "],join_type='" + join_type + "')"
                    query = eval(q)

            str_query = query.to_string()
            return query

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def build_view_league(self, main_feature_group, secondary_feature_groups, view_name, view_version, labels,
                          description):
        try:
            logger.info('Getting feature groups details')
            fss_obj = fss.FeatureStoreService(self.conf)
            main_feature_group_handle = fss_obj.get_group(main_feature_group["name"], main_feature_group["version"])
            if len(main_feature_group["filters"]) > 0:
                main_feature_group_handle = main_feature_group_handle.filter(eval(main_feature_group["filters"]))

            query = "main_feature_group_handle.select(main_feature_group['features'])"
            for group in secondary_feature_groups:
                name = group["name"]
                version = group["version"]
                secondary_feature_group_handle = fss_obj.get_group(name, version)
                if len(group["filters"]) > 0:
                    secondary_feature_group_handle = secondary_feature_group_handle.filter(eval(group["filters"]))
                query = query + ".join(secondary_feature_group_handle.select(group['features']), \
                          on=main_feature_group['joining_column']==group['joining_column'], \
                          join_type=group['join_type'])"
            query_string = eval(query)

            fss_obj.build_view(view_name, view_version, description, "", query_string)

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def build_view(self, view_name, view_version, description, view_json):
        try:
            cfs_obj = ComplexFeatureService(self.conf)
            if (view_name == 'team'):
                ##TODO: Implement logic for team view
                pass
                # query = cfs_obj.build_view_team()
                # fss_obj = fss.FeatureStoreService(self.conf)
                # fss_obj.build_view(view_name, view_version, description, "", query)

            elif (view_name == 'league'):
                ##TODO: Implement logic for league view
                pass
                # query = cfs_obj.build_view_league(view_name, view_version, description, view_json)
                # fss_obj = fss.FeatureStoreService(self.conf)
                # fss_obj.build_view(view_name, view_version, description, "", query)

            elif (view_name == 't1vst2'):
                query = cfs_obj.build_view_t1vst2(view_name, view_version, description, view_json)
                fss_obj = fss.FeatureStoreService(self.conf)
                fss_obj.build_view(view_name, view_version, description, "", query)
            else:
                logger.error('Feature view function not found. Please implement the function in '
                             'ComplexFeatureService class.')
        except Exception as e:
            logger.error(e)

        finally:
            pass


import logging
class process_logging:

    def process_log(self, logger_name, filename):
        logger = logging.getLogger(logger_name)
        formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(message)s')
        fileHandler = logging.FileHandler(filename, mode='a')
        fileHandler.setFormatter(formatter)
        streamHandler = logging.StreamHandler()
        streamHandler.setFormatter(formatter)
        logger.setLevel(logging.INFO)
        logger.addHandler(fileHandler)
        logger.addHandler(streamHandler)



import pandas as pd
from itertools import cycle

def is_scalar(obj):
    if isinstance(obj, str):
        return True
    elif hasattr(obj, "__iter__"):
        return False
    else:
        return True

def multi_melt(
        df: pd.DataFrame,
        id_vars=None,
        value_vars=None,
        var_name=None,
        value_name="value",
        col_level=None,
        ignore_index=True,
    ) -> pd.DataFrame:
    value_vars = value_vars if not is_scalar(value_vars[0]) else [value_vars]
    var_name = var_name if not is_scalar(var_name) else cycle([var_name])
    value_name = value_name if not is_scalar(value_name) else cycle([value_name])

    melted_dfs = [
        (
            df.melt(
                id_vars,
                *melt_args,
                col_level,
                ignore_index,
            ).pipe(lambda df: df.set_index([*id_vars, df.groupby(id_vars).cumcount()]))
        )
        for melt_args in zip(value_vars, var_name, value_name)
    ]

    return (
        pd.concat(melted_dfs, axis=1)
        .reset_index()
    )

def call_multi_melt(df: pd.DataFrame, id_vars: list, value_vars: list, value_name: list):
    df_melted = df.pipe(
        multi_melt,
        id_vars=id_vars,
        value_vars=value_vars,
        value_name=value_name,
    )
    return df_melted


import json
import logging
from sqlalchemy import create_engine
import pandas as pd

logger = logging.getLogger('logger')


class DbTables:

    def __init__(self, config):
        self.conf = config
        with open(self.conf, "r") as jsonfile:
            data = json.load(jsonfile)

        self.db_host = data['host']
        self.db_name = data['database']
        self.db_user = data['username']
        self.db_password = data['password']

    def get_db_conn(self):
        try:
            logger.info('Trying to establish connection with the database')
            engine = create_engine(f"mysql+pymysql://{self.db_user}:{self.db_password}@{self.db_host}/{self.db_name}",
                                   echo=True)
            conn = engine.connect()
            return conn

        except Exception as e:
            logger.error(e)

        finally:
            logger.info('Database connection successful')

    def close_db_conn(self, conn):
        try:
            conn.close()

        except Exception as e:
            logger.error(e)

        finally:
            logger.info('Database connection closed')

    def run_query(self, conn, query):
        try:
            logger.info('Running query in the database')
            results = conn.execute(query)
            df = pd.DataFrame(results.fetchall())
            return df

        except Exception as e:
            logger.error(e)

        finally:
            logger.info('Query executed successfully')


import unittest
import sys
import os
import json

parent_dir = os.path.dirname(__file__)
sys.path.insert(0, parent_dir)
from src.database import DbTables
from src.service import FeatureStoreService

"""
UnitTest class: All the test cases are to be written here and this test class can be called using the following command:
pytest -v UnitTest.py
"""

class Tests(unittest.TestCase):
    config = 'D:\\Documents\\Office Documents\\Toptal\\TSGNC\\basketball-feature-tool\\tests\\test_conf.json'

    def test_db_connectivity(self):
        db = DbTables.DbTables(self.config)
        conn = db.get_db_conn()
        query = 'select 1 as test_col'
        actual_df = DbTables.DbTables.run_query(self, conn, query)
        returned_value = actual_df.iloc[0, 0]
        expected_value = 1

        self.assertTrue(returned_value == expected_value)

    def test_hopsworks_connectivity(self):
        with open(self.config, "r") as jsonfile:
            test_data = json.load(jsonfile)
        fss = FeatureStoreService.FeatureStoreService(self.config)
        fs = fss.get_feature_store_connection()
        expected_value = test_data['hopsworks_project'] + '_featurestore'
        returned_value = fs.name

        self.assertTrue(returned_value == expected_value)

    def test_hopsworks_group_retrieval(self):
        with open(self.config, "r") as jsonfile:
            test_data = json.load(jsonfile)
        test_group_name = test_data['test_group_name']
        test_group_version = test_data['test_group_version']
        fss = FeatureStoreService.FeatureStoreService(self.config)
        fg = fss.get_group(test_group_name, test_group_version)

        expected_value = test_group_name
        returned_value = fg.name

        self.assertTrue(returned_value == expected_value)

    def test_hopsworks_view_retrieval(self):
        with open(self.config, "r") as jsonfile:
            test_data = json.load(jsonfile)
        test_view_name = test_data['test_view_name']
        test_view_version = test_data['test_view_version']
        fss = FeatureStoreService.FeatureStoreService(self.config)
        fv = fss.get_view(test_view_name, test_view_version)

        expected_value = test_view_name
        returned_value = fv.name

        self.assertTrue(returned_value == expected_value)



import json
import logging
import pandas as pd
from sqlalchemy.sql import text
from pathlib import Path
import sys
import os

parent_dir = os.path.dirname(os.getcwd())
sys.path.insert(0,parent_dir)

from src.service import FeatureStoreService as fss, ComplexFeatureService as cfs
from src.database import DbTables

logger = logging.getLogger('logger')

class SimpleFeatureService:
    def __init__(self, config):
        self.conf = config
        with open(self.conf, "r") as jsonfile:
            data = json.load(jsonfile)

    def get_group(self, group_name: str, version: int):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fg = fss_obj.get_group(group_name, version)
            return fg

        except Exception as e:
            logger.error(e)

    def get_group_features(self, group_name: str, version: int, features):
        """

        :param group_name: Name of the feature group required to be fetched
        :param version: Version of the feature group required to be fetched
        :param features: Comma-separated list of features of the feature group required to be fetched
        :return: returns dataframe of required features
        """
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fg = fss_obj.get_group(group_name, version)
            query = fg.select(features)
            df = query.read()
            print(df.head())
            return df

        except Exception as e:
            logger.error(e)

    def build_group_from_db(self, group_name: str, version: int, description: str, table_name: str, primary_key: [],
                            partition_key: [], features: [], derived_features: []):
        """

        :param group_name: Name of the feature group to be built
        :param version: Version of the feature group to be built
        :param description: Short description of the feature group to be built
        :param table_name: Name of the table name which would be used to create the feature group
        :param primary_key: List of columns that would be designated as prime attributes in the feature group
        :param partition_key: List of columns that would be designated as partition key in the feature group
        :param features: List of table columns that would be used as features in the feature group
        :param derived_features: List of derived columns from the table that would be used as features in the feature group
        :return: returns nothing
        """
        try:
            dbtables_obj = DbTables.DbTables(self.conf)
            conn = dbtables_obj.get_db_conn()
            logger.info('Getting table details and data from the database')
            query = text('SELECT ' + ','.join(features + derived_features).strip(',') + ' FROM ' + table_name)
            df = dbtables_obj.run_query(conn, query)
            fss_obj = fss.FeatureStoreService(self.conf)
            fss_obj.build_group(group_name, version, description, primary_key, partition_key, features, df)

        except Exception as e:
            logger.error(e)

        finally:
            dbtables_obj.close_db_conn(conn)

    def build_group_from_view(self, group_name: str, group_version: int, description: str, view_name: str,
                              view_version: int, features: [],
                              primary_key: [], partition_key: []):
        try:
            logger.info('Getting details from the feature view')
            fss_obj = fss.FeatureStoreService(self.conf)
            fv = fss_obj.get_view(view_name, view_version)
            df = fv.query.read()
            df_filtered = df[[x.lower() for x in features]]
            fss_obj.build_group(group_name, group_version, description, primary_key, partition_key, df_filtered.columns.tolist(),
                                df_filtered)

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def build_group_from_file(self, group_name: str, group_version: int, description: str, file_path: str,
                              file_name: str, sep: str, index: str, header: str, quote_char: str, escape_char: str,
                              primary_key: [], partition_key: [], features: []):
        try:
            logger.info('Getting details from the file')
            fss_obj = fss.FeatureStoreService(self.conf)
            df = pd.read_csv(file_path + file_name, sep=sep, index_col=eval(index), header=header, quotechar=quote_char,
                             escapechar=escape_char)
            df_filtered = df[[x.lower() for x in features]]
            fss_obj.build_group(group_name, group_version, description, primary_key, partition_key, df_filtered.columns.tolist(),
                                df_filtered)

        except Exception as e:
            logger.error(e)

        finally:
            pass

    def export_group_to_file(self, group_name: str, version: int, file_path: str, file_name: str):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fg = fss_obj.get_group(group_name, version)
            query = fg.select_all()
            df = query.read()

            out_file = file_name
            out_dir = Path(file_path)
            out_dir.mkdir(parents=True, exist_ok=True)
            df.to_csv(out_dir / out_file, sep='|', index=False, header=True, quotechar='"', escapechar="\"")

            return fg

        except Exception as e:
            logger.error(e)

    def drop_group(self, group_name: str, version: int):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fss_obj.drop_group(group_name, version)
        except Exception as e:
            logger.error(e)

    def get_view(self, view_name: str, version: int):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fv = fss_obj.get_view(view_name, version)
            return fv

        except Exception as e:
            logger.error(e)

    def get_view_features(self, view_name: str, version, features):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fv = fss_obj.get_view(view_name, version)
            df = fv.query.read()
            df_filtered = df[features]
            print(df_filtered.head())
            return df_filtered

        except Exception as e:
            logger.error(e)

    def build_view(self, view_name, view_version, description, view_json):
        try:
            cfs_obj = cfs.ComplexFeatureService(self.conf)

            if (view_name == 'view_name1'):
                ##TODO: Implement logic for view 1
                pass

            elif (view_name == 'view_name2'):
                ##TODO: Implement logic for view 2
                pass

            elif (view_name == 'view_name3'):
                ##TODO: Implement logic for view 3
                pass
            else:
                logger.error('Feature view function not found. Please implement the function in '
                             'ComplexFeatureService class.')
        except Exception as e:
            logger.error(e)

        finally:
            pass

    def export_view_to_file(self, view_name: str, version: int, file_path: str, file_name: str):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fv = fss_obj.get_view(view_name, version)

            df = fv.query.read()

            out_file = file_name
            out_dir = Path(file_path)
            out_dir.mkdir(parents=True, exist_ok=True)
            df.to_csv(out_dir / out_file, sep='|', index=False, header=True, quotechar='"', escapechar="\"")

            return fv

        except Exception as e:
            logger.error(e)

    def drop_view(self, view_name: str, version: int):
        try:
            fss_obj = fss.FeatureStoreService(self.conf)
            fss_obj.drop_view(view_name, version)
        except Exception as e:
            logger.error(e)



import json
import logging
import argparse
from datetime import datetime
from src.service import SimpleFeatureService, ComplexFeatureService
from src.utils import process_logging
from pathlib import Path
import sys

def main(argv=sys.argv[1:]):
    try:
        """
            There are 12 run modes to call this application from the CLI.
            1) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode get_group --group_name gp_game_base --group_version 1
            2) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode get_group_features --group_name gp_box_score --group_version 1 --features adv_idx_uuid,box_type,mp
            3) python.exe cmd.py --conf "/path/to/config/conf.json" --build_conf "/path/to/config/groups.json" --run_mode build_group_from_db --group_name gp_game_base
            4) python.exe cmd.py --conf "/path/to/config/conf.json" --build_conf "/path/to/config/groups.json" --run_mode build_group_from_view --group_name test_group_from_view
            5) python.exe cmd.py --conf "/path/to/config/conf.json" --build_conf "/path/to/config/groups.json" --run_mode build_group_from_file --group_name test_group_from_file
            6) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode export_group_to_file --group_name gp_game_base --group_version 1 --file_path "/path/to/export/folder/" --file_name gp_game_base_export.csv
            7) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode drop_group --group_name gp_game_base --group_version 1
            8) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode get_view --view_name t1vst2 --view_version 1
            9) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode get_view_features --view_name t1vst2 --view_version 1  --features uuid,box_type,mp
            10) python.exe cmd.py --conf "/path/to/config/conf.json" --build_conf "/path/to/config/views.json" --run_mode build_view --view_name t1vst2
            11) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode export_view_to_file --view_name t1vst2 --view_version 1 --file_path "/path/to/export/folder/" --file_name t1vst2_view_export.csv
            12) python.exe cmd.py --conf "/path/to/config/conf.json" --run_mode drop_view --view_name t1vst2 --view_version 1
    
            python.exe cmd.py --conf "config/conf.json" --build_conf "config/groups.json" --run_mode build_group --group_name gp_game_base
    
            Mandatory arguments: 
                --conf, --run_mode
            Optional arguments:
                build_conf, group_name, group_version, view_name, view_version, file_path, and file_name are optional and 
                will be used based on the operation type. For e.g. if you are running only get_group then you don't need 
                to specify build_conf argument, but you when you are running build_group, then you need to specify 
                build_conf argument but you don't need to specify group version in that. 
                See the examples below for clarification.
        """
        parser = argparse.ArgumentParser(description="Arguments for calling Feature Store API")

        parser.add_argument('-cn', '--conf',
                            type=str,
                            default='config/conf.json',
                            help='name of the configuration file',
                            required=True)

        parser.add_argument('-bc', '--build_conf',
                            type=str,
                            default='config/groups.json',
                            help='name of the preset file required to create a group or a view',
                            required=False)

        parser.add_argument('-r', '--run_mode',
                            type=str,
                            default='get_group',
                            help='type of operations: '
                                 '1) get_group'
                                 '2) get_group_features'
                                 '3) build_group_from_db'
                                 '4) build_group_from_view'
                                 '5) build_group_from_file'
                                 '6) export_group_to_file'
                                 '7) drop_group'
                                 '8) get_view'
                                 '9) get_view_features'
                                 '10) build_view'
                                 '11) export_view_to_file'
                                 '12) drop_view',
                            required=True)

        parser.add_argument('-gn', '--group_name',
                            type=str,
                            default='gp_game_base',
                            help='name of the required feature group',
                            required=False)

        parser.add_argument('-gv', '--group_version',
                            type=str,
                            default='1',
                            help='version of the feature group required to retrieve or create',
                            required=False)

        parser.add_argument('-vn', '--view_name',
                            type=str,
                            default='t1vst2',
                            help='name of the required feature view',
                            required=False)

        parser.add_argument('-vv', '--view_version',
                            type=str,
                            default='1',
                            help='version of the feature version required to retrieve or create',
                            required=False)

        parser.add_argument('-f', '--features',
                            type=str,
                            default='',
                            help='comma separated list of required group or view features',
                            required=False)

        parser.add_argument('-fp', '--file_path',
                            type=str,
                            default='files',
                            help='directory of the input/output files',
                            required=False)

        parser.add_argument('-fn', '--file_name',
                            type=str,
                            default='export.csv',
                            help='name of the input/output files',
                            required=False)

        args = parser.parse_args(argv)

        with open(args.conf, "r") as jsonfile:
            conf = json.load(jsonfile)

        today_datetime = datetime.now().strftime("%d%m%Y_%H%M%S")
        Path(conf['log_path']).mkdir(parents=True, exist_ok=True)
        process_logger = process_logging.process_logging()
        process_logger.process_log('logger', conf['log_path'] + '\\logs_' + today_datetime + '.txt')
        logger = logging.getLogger('logger')

        logger.info('The job has started')

        sfs = SimpleFeatureService.SimpleFeatureService(args.conf)
        cfs = ComplexFeatureService.ComplexFeatureService(args.conf)

        if args.run_mode == 'get_group':
            fg = sfs.get_group(args.group_name, args.group_version)
            return fg

        elif args.run_mode == 'get_group_features':
            df = sfs.get_group_features(args.group_name, args.group_version, list(args.features.split(',')))
            return df

        elif args.run_mode == 'build_group_from_db':
            with open(args.build_conf, "r") as jsonfile:
                groups_info = json.load(jsonfile)
                if args.group_name not in groups_info:
                    logger.error('Feature group key is not found in the groups json file')

                for group_name in groups_info:
                    if group_name == args.group_name:
                        group = groups_info[group_name]
                        sfs.build_group_from_db(group['hopsworks_group_name'],
                                                group['hopsworks_group_version'],
                                                group['description'],
                                                group['table_name'],
                                                group['primary_key'],
                                                group['partition_key'],
                                                group['features'],
                                                group['derived_features'])

        elif args.run_mode == 'build_group_from_view':
            with open(args.build_conf, "r") as jsonfile:
                groups_info = json.load(jsonfile)
                if args.group_name not in groups_info:
                    logger.error('Feature group key is not found in the groups json file')

                for group_name in groups_info:
                    if group_name == args.group_name:
                        group = groups_info[group_name]
                        sfs.build_group_from_view(group['hopsworks_group_name'],
                                                  group['hopsworks_group_version'],
                                                  group['description'],
                                                  group['hopsworks_view_name'],
                                                  group['hopsworks_view_version'],
                                                  group['features'],
                                                  group['primary_key'],
                                                  group['partition_key'])

        elif args.run_mode == 'build_group_from_file':
            with open(args.build_conf, "r") as jsonfile:
                groups_info = json.load(jsonfile)
                if args.group_name not in groups_info:
                    logger.error('Feature group key is not found in the groups json file')

                for group_name in groups_info:
                    if group_name == args.group_name:
                        group = groups_info[group_name]
                        sfs.build_group_from_file(group['hopsworks_group_name'],
                                                  group['hopsworks_group_version'],
                                                  group['description'],
                                                  group['file_path'],
                                                  group['file_name'],
                                                  group['sep'],
                                                  group['index_col'],
                                                  group['header_row'],
                                                  group['quote_char'],
                                                  group['escape_char'],
                                                  group['primary_key'],
                                                  group['partition_key'],
                                                  group['features'])

        elif args.run_mode == 'export_group_to_file':
            sfs.export_group_to_file(args.group_name, args.group_version, args.file_path, args.file_name)

        elif args.run_mode == 'drop_group':
            sfs.drop_group(args.group_name, args.group_version)

        elif args.run_mode == 'get_view':
            fv = sfs.get_view(args.view_name, args.view_version)
            return fv

        elif args.run_mode == 'get_view_features':
            df = sfs.get_view_features(args.view_name, args.view_version, list(args.features.split(',')))
            return df

        elif args.run_mode == 'build_view':
            with open(args.build_conf, "r") as jsonfile:
                views_info = json.load(jsonfile)
                if args.view_name not in views_info:
                    logger.error('Feature group key is not found in the groups json file')

                for view_name in views_info:
                    if view_name == args.view_name:
                        if (args.view_name in ['t1vst2', 'team', 'league']):
                            view = views_info[view_name]
                            a = view['hopsworks_view_name']
                            cfs.build_view(view['hopsworks_view_name'],
                                           view['hopsworks_view_version'],
                                           view['description'],
                                           view['features'])
                        else:
                            view = views_info[view_name]
                            a = view['hopsworks_view_name']
                            sfs.build_view(view['hopsworks_view_name'],
                                           view['hopsworks_view_version'],
                                           view['description'],
                                           view['features'])
        elif args.run_mode == 'export_view_to_file':
            sfs.export_view_to_file(args.view_name, args.view_version, args.file_path, args.file_name)

        elif args.run_mode == 'drop_view':
            sfs.drop_view(args.view_name, args.view_version)

    except Exception as e:
        logger.error(e)


if __name__ == '__main__':
    main(sys.argv[1:])


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Hopsworks_Integration", # Replace with your own username
    version="0.0.1",
    author="Syed Muneeb Hussain",
    author_email="muneebhussain94@gmail.com",
    description="A python package that exposes functions to interact with the Hopsworks feature store",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TSGNC/basketball-feature-tool",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7,<3.10',
    install_requires=[
        'hopsworks==3.0.3',
        'pandas',
        'SQLAlchemy',
        'pytest'
    ]
)